@@ -13,17 +13,19 @@ This module contains base functions and classes XML Schema components.
 import re
 from typing import TYPE_CHECKING, cast, Any, Dict, Generic, List, Iterator, Optional, \
     Set, Tuple, TypeVar, Union, MutableMapping
+from xml.etree import ElementTree
 
-import elementpath
+from elementpath import select
+from elementpath.etree import is_etree_element, etree_tostring
 
 from ..exceptions import XMLSchemaValueError, XMLSchemaTypeError
-from ..names import XSD_ANNOTATION, XSD_APPINFO, XSD_DOCUMENTATION, XML_LANG, \
-    XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE, XSD_ID, XSD_QNAME, \
-    XSD_OVERRIDE, XSD_NOTATION_TYPE, XSD_DECIMAL
-from ..etree import is_etree_element, etree_tostring, etree_element
+from ..names import XSD_ANNOTATION, XSD_APPINFO, XSD_DOCUMENTATION, \
+    XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE, XSD_ID, \
+    XSD_QNAME, XSD_OVERRIDE, XSD_NOTATION_TYPE, XSD_DECIMAL
 from ..aliases import ElementType, NamespacesType, SchemaType, BaseXsdType, \
     ComponentClassType, ExtraValidatorType, DecodeType, IterDecodeType, \
     EncodeType, IterEncodeType
+from ..translation import gettext as _
 from ..helpers import get_qname, local_name, get_prefixed_qname
 from ..resources import XMLResource
 from .exceptions import XMLSchemaParseError, XMLSchemaValidationError
@@ -47,8 +49,8 @@ Ref.: https://www.w3.org/TR/xmlschema11-1/#key-va
 
 def check_validation_mode(validation: str) -> None:
     if validation not in XSD_VALIDATION_MODES:
-        raise XMLSchemaValueError("validation mode can be 'strict', "
-                                  "'lax' or 'skip': %r" % validation)
+        raise XMLSchemaValueError(_("validation mode can be 'strict', "
+                                    "'lax' or 'skip': %r") % validation)
 
 
 class XsdValidator:
@@ -66,7 +68,7 @@ class XsdValidator:
     :ivar errors: XSD validator building errors.
     :vartype errors: list
     """
-    elem: Optional[etree_element] = None
+    elem: Optional[ElementTree.Element] = None
     namespaces: Any = None
     errors: List[XMLSchemaParseError]
 
@@ -250,8 +252,9 @@ class XsdValidator:
             return value
         else:
             admitted_values = ('##defaultNamespace', '##targetNamespace', '##local')
-            msg = "wrong value %r for 'xpathDefaultNamespace' attribute, can be (anyURI | %s)."
-            self.parse_error(msg % (value, ' | '.join(admitted_values)), elem)
+            msg = _("wrong value {0!r} for 'xpathDefaultNamespace' "
+                    "attribute, can be (anyURI | {1}).")
+            self.parse_error(msg.format(value, ' | '.join(admitted_values)), elem)
             return ''
 
 
@@ -273,7 +276,7 @@ class XsdComponent(XsdValidator):
     _REGEX_SPACES = re.compile(r'\s+')
     _ADMITTED_TAGS: Union[Set[str], Tuple[str, ...], Tuple[()]] = ()
 
-    elem: etree_element
+    elem: ElementTree.Element
     parent = None
     name = None
     ref: Optional['XsdComponent'] = None
@@ -282,7 +285,7 @@ class XsdComponent(XsdValidator):
     _annotation = None
     _target_namespace: Optional[str]
 
-    def __init__(self, elem: etree_element,
+    def __init__(self, elem: ElementTree.Element,
                  schema: SchemaType,
                  parent: Optional['XsdComponent'] = None,
                  name: Optional[str] = None) -> None:
@@ -400,15 +403,17 @@ class XsdComponent(XsdValidator):
             if 'name' in self.elem.attrib:
                 return None
             elif self.parent is None:
-                self.parse_error("missing attribute 'name' in a global %r" % type(self))
+                msg = _("missing attribute 'name' in a global %r")
+                self.parse_error(msg % type(self))
             else:
-                self.parse_error(
-                    "missing both attributes 'name' and 'ref' in local %r" % type(self)
-                )
+                msg = _("missing both attributes 'name' and 'ref' in local %r")
+                self.parse_error(msg % type(self))
         elif 'name' in self.elem.attrib:
-            self.parse_error("attributes 'name' and 'ref' are mutually exclusive")
+            msg = _("attributes 'name' and 'ref' are mutually exclusive")
+            self.parse_error(msg)
         elif self.parent is None:
-            self.parse_error("attribute 'ref' not allowed in a global %r" % type(self))
+            msg = _("attribute 'ref' not allowed in a global %r")
+            self.parse_error(msg % type(self))
         else:
             try:
                 self.name = self.schema.resolve_qname(ref)
@@ -416,8 +421,8 @@ class XsdComponent(XsdValidator):
                 self.parse_error(err)
             else:
                 if self._parse_child_component(self.elem, strict=False) is not None:
-                    self.parse_error("a reference component cannot have "
-                                     "child definitions/declarations")
+                    msg = _("a reference component cannot have child definitions/declarations")
+                    self.parse_error(msg)
                 return True
 
         return None
@@ -431,7 +436,7 @@ class XsdComponent(XsdValidator):
             elif not strict:
                 return e
             elif child is not None:
-                msg = "too many XSD components, unexpected {!r} found at position {}"
+                msg = _("too many XSD components, unexpected {0!r} found at position {1}")
                 self.parse_error(msg.format(child, elem[:].index(e)), elem)
                 break
             else:
@@ -447,23 +452,26 @@ class XsdComponent(XsdValidator):
 
         self._target_namespace = self.elem.attrib['targetNamespace'].strip()
         if 'name' not in self.elem.attrib:
-            self.parse_error("attribute 'name' must be present when "
-                             "'targetNamespace' attribute is provided")
+            msg = _("attribute 'name' must be present when "
+                    "'targetNamespace' attribute is provided")
+            self.parse_error(msg)
         if 'form' in self.elem.attrib:
-            self.parse_error("attribute 'form' must be absent when "
-                             "'targetNamespace' attribute is provided")
+            msg = _("attribute 'form' must be absent when "
+                    "'targetNamespace' attribute is provided")
+            self.parse_error(msg)
         if self._target_namespace != self.schema.target_namespace:
             if self.parent is None:
-                self.parse_error("a global %s must have the same namespace as "
-                                 "its parent schema" % self.__class__.__name__)
+                msg = _("a global %s must have the same namespace as its parent schema")
+                self.parse_error(msg % self.__class__.__name__)
 
             xsd_type = self.get_parent_type()
             if xsd_type is None or xsd_type.parent is not None:
                 pass
             elif xsd_type.derivation != 'restriction' or \
                     getattr(xsd_type.base_type, 'name', None) == XSD_ANY_TYPE:
-                self.parse_error("a declaration contained in a global complexType "
-                                 "must have the same namespace as its parent schema")
+                msg = _("a declaration contained in a global complexType "
+                        "must have the same namespace as its parent schema")
+                self.parse_error(msg)
 
         if self.name is None:
             pass  # pragma: no cover
@@ -581,7 +589,8 @@ class XsdComponent(XsdValidator):
                 return component
             component = component.parent
         else:
-            raise XMLSchemaValueError(f"parent circularity from {self}")  # pragma: no cover
+            msg = _("parent circularity from {}")
+            raise XMLSchemaValueError(msg.format(self))  # pragma: no cover
 
     def get_parent_type(self) -> Optional['XsdType']:
         """
@@ -664,7 +673,7 @@ class XsdAnnotation(XsdComponent):
         return '%s(%r)' % (self.__class__.__name__, str(self)[:40])
 
     def __str__(self) -> str:
-        return '\n'.join(elementpath.select(self.elem, '*/fn:string()'))
+        return '\n'.join(select(self.elem, '*/fn:string()'))
 
     @property
     def built(self) -> bool:
@@ -675,14 +684,8 @@ class XsdAnnotation(XsdComponent):
         self.documentation = []
         for child in self.elem:
             if child.tag == XSD_APPINFO:
-                for key in child.attrib:
-                    if key != 'source':
-                        self.parse_error("wrong attribute %r for appinfo declaration." % key)
                 self.appinfo.append(child)
             elif child.tag == XSD_DOCUMENTATION:
-                for key in child.attrib:
-                    if key not in ['source', XML_LANG]:
-                        self.parse_error("wrong attribute %r for documentation declaration." % key)
                 self.documentation.append(child)
 
 
@@ -894,8 +897,8 @@ class ValidationMixin(Generic[ST, DT]):
         validations on XML data. The provided function is called for each traversed \
         element, with the XML element as 1st argument and the corresponding XSD \
         element as 2nd argument. It can be also a generator function and has to \
-        raise/yield :exc:`XMLSchemaValidationError` exceptions.
-        :raises: :exc:`XMLSchemaValidationError` if the XML data instance is invalid.
+        raise/yield :exc:`xmlschema.XMLSchemaValidationError` exceptions.
+        :raises: :exc:`xmlschema.XMLSchemaValidationError` if the XML data instance is invalid.
         """
         for error in self.iter_errors(obj, use_defaults, namespaces,
                                       max_depth, extra_validator):
@@ -953,7 +956,7 @@ class ValidationMixin(Generic[ST, DT]):
         a simple data type object otherwise. If *validation* argument is 'lax' a 2-items \
         tuple is returned, where the first item is the decoded object and the second item \
         is a list containing the errors.
-        :raises: :exc:`XMLSchemaValidationError` if the object is not decodable by \
+        :raises: :exc:`xmlschema.XMLSchemaValidationError` if the object is not decodable by \
         the XSD component, or also if it's invalid when ``validation='strict'`` is provided.
         """
         check_validation_mode(validation)
@@ -981,8 +984,8 @@ class ValidationMixin(Generic[ST, DT]):
         a string if it's simple type datum. If *validation* argument is 'lax' a 2-items \
         tuple is returned, where the first item is the encoded object and the second item \
         is a list containing the errors.
-        :raises: :exc:`XMLSchemaValidationError` if the object is not encodable by the XSD \
-        component, or also if it's invalid when ``validation='strict'`` is provided.
+        :raises: :exc:`xmlschema.XMLSchemaValidationError` if the object is not encodable by \
+        the XSD component, or also if it's invalid when ``validation='strict'`` is provided.
         """
         check_validation_mode(validation)
         result, errors = None, []
